package ruon.jiao;

import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.util.*;
import java.io.ByteArrayInputStream;
import java.io.IOException;

public class Config {
    private Map agent;
    private List resources;

    static public class Resource {
        private Map params;
        private Resource(Map params) {
            this.params = params;
        }
        String get(String name) {
            return (String) params.get(name);
        }
    }

    public String get(String name) {
        return (String) agent.get(name);
    }
    public Iterator resourceIterator() {
        return resources.iterator();
    }

    Config(String data) throws JIAOException, IOException {
        try {
            DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            load(builder.parse(new ByteArrayInputStream(data.getBytes())));
        } catch (SAXException e) {
            throw new JIAOException("Error parsing config data", e);
        } catch (ParserConfigurationException e) {
            throw new JIAOException("Error Parsing Config data", e);
        }
    }

    private void load(Document document) throws JIAOException {
        agent = new HashMap();
        resources = new ArrayList();

        Node n = document.getFirstChild();
        if (!"config".equals(n.getNodeName())) {
            throw new JIAOException("Expected config node but got "+n.getNodeName());
        }
        NodeList nodeList = n.getChildNodes();
        for (int i=0;i<nodeList.getLength();++i) {
            n = nodeList.item(i);
            if ("resources".equals(n.getNodeName())) {
                loadResources(n);
            } else {
                agent.put(n.getNodeName(), n.getTextContent());
            }
        }

    }
    private void loadResources(Node n) {
        NodeList nodeList = n.getChildNodes();
        for (int i=0;i<nodeList.getLength();++i) {
            n = nodeList.item(i);
            resources.add(loadResource(n));
        }
    }

    private Resource loadResource(Node n) {
        Map params = new HashMap();
        NodeList nodeList = n.getChildNodes();
        for (int i=0;i<nodeList.getLength();++i) {
            n = nodeList.item(i);
            params.put(n.getNodeName(), n.getTextContent());
        }
        return new Resource(params);
    }
}
